﻿--[[
	LootMaster player/candidate Stuff
]]--

local INVTYPE_Slots = {
		INVTYPE_HEAD		    = "HeadSlot",
		INVTYPE_NECK		    = "NeckSlot",
		INVTYPE_SHOULDER	    = "ShoulderSlot",
		INVTYPE_CLOAK		    = "BackSlot",
		INVTYPE_CHEST		    = "ChestSlot",
		INVTYPE_WRIST		    = "WristSlot",
		INVTYPE_HAND		    = "HandsSlot",
		INVTYPE_WAIST		    = "WaistSlot",
		INVTYPE_LEGS		    = "LegsSlot",
		INVTYPE_FEET		    = "FeetSlot",
		INVTYPE_SHIELD		    = "SecondaryHandSlot",
		INVTYPE_ROBE		    = "ChestSlot",
		INVTYPE_2HWEAPON	    = {"MainHandSlot","SecondaryHandSlot"},
		INVTYPE_WEAPONMAINHAND	= "MainHandSlot",
		INVTYPE_WEAPONOFFHAND	= {"SecondaryHandSlot",["or"] = "MainHandSlot"},
		INVTYPE_WEAPON		    = {"MainHandSlot","SecondaryHandSlot"},
		INVTYPE_THROWN		    = "RangedSlot",
		INVTYPE_RANGED		    = "RangedSlot",
		INVTYPE_RANGEDRIGHT 	= "RangedSlot",
		INVTYPE_FINGER		    = {"Finger0Slot","Finger1Slot"},
		INVTYPE_HOLDABLE	    = {"SecondaryHandSlot", ["or"] = "MainHandSlot"},
		INVTYPE_TRINKET		    = {"TRINKET0SLOT", "TRINKET1SLOT"}
}

local lm 	= LootMaster		-- Local instance of the addon

function LootMaster:Debug( message )
    if not self.debug then return end;
    self:Print("debug: " .. message)    
end

function LootMaster:SendCommand(command, message, target)
	if not target then
		return self:Print("Could not send command, no target specified")
	end;
    
    if UnitInRaid(target) then
        -- we're in raid with master looter
        self:SendCommMessage("EPGPLootMasterML", format("%s:%s", tostring(command), tostring(message)), "RAID", nil, "ALERT")
    elseif UnitInParty(target) and GetNumPartyMembers()>0 then
        --we're in party with master looter
        self:SendCommMessage("EPGPLootMasterML", format("%s:%s", tostring(command), tostring(message)), "PARTY", nil, "ALERT")
    else
        --we're not grouped, send message to target by whispering
        self:SendCommMessage("EPGPLootMasterML", format("%s:%s", tostring(command), tostring(message)), "WHISPER", target, "ALERT")
    end	
end

--[[
	Event gets triggered when candidate receives a message from the Masterlooter
]]
function LootMaster:CommandReceived(prefix, message, distribution, sender)
	local _,_,command, message = string.find(message, "^([%a_]-):(.*)$")
	command = strupper(command or '');
	message = message or '';	

	if command == 'DO_YOU_WANT' then

		-- Masterlooter wants to know from us if we'd like to have the item.
		-- Lets show the gui and ask the player for input.

		local itemID, gpvalue, ilevel, gpvalue2, binding, slot, quality, timeout, link, texture = strsplit("^", message)

		-- local _, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture = GetItemInfo(link)
        
        -- Send the master loot our current gear and version number
		self:SendCommand( 'GEAR', format('%s^%s^%s', itemID, self.iVersion or 0, GetGearByINVTYPE(slot)), sender)
        
        if not self:HasLoot(link) then
            
            -- add the loot to the lootlist and redraw the ui
            tinsert( self.lootList, {
                ["lootmaster"]  = sender,
                ["link"]        = link,
                ["id"]          = itemID,
                ["ilevel"]      = ilevel,
                ["gpvalue"]     = gpvalue,
                ["gpvalue2"]    = gpvalue2,
                ["binding"]     = binding,
                ["slot"]        = slot,
                ["texture"]     = texture,
                ["timeout"]     = tonumber(timeout),
                ["timeoutLeft"] = tonumber(timeout),
                ["quality"]     = tonumber(quality)
            });
            
            self.lootMLCache[link] = sender;
            
            self:UpdateLootUI();
        
        end
        
        
    elseif command == 'DISCARD' then
        
        -- Message gets received
        local itemID, link = strsplit("^", message)
        
        if self:RemoveLoot(link) then
            self:UpdateLootUI();
        end
        
    elseif command == 'LOOTED' then
        -- Someone looted an item through lootmaster.
        -- Update the UI and send some info to ct_raidtracker if its active.
        
        -- Message gets received
        local player, link, lootType, lootGP = strsplit("^", message)
        
        self:Debug('looted: ' .. message)
        
        if self.lootMLCache[link] and self.lootMLCache[link] ~= sender then
            return self:Print(format('%s sent a message that %s has been looted, but is not lootmaster for this item (his version outdated?). Message ignored', sender, link))
        end
        self.lootMLCache[link] = nil;
        
        if self:RemoveLoot(link) then
            self:UpdateLootUI();
        end
        
        self:Debug('UI updated');
        
        if not player or not link then
            return self:Debug('!player or !link')
        end;
        
        lootTypeID = tonumber(lootType) or LootMaster.LOOTTYPE.UNKNOWN;
        lootType = LootMaster.LOOTTYPE[lootTypeID] or LootMaster.LOOTTYPE[LootMaster.LOOTTYPE.UNKNOWN];
        lootGP = tonumber(lootGP) or -1;
        
        self:Debug('before cttracker register')
        local debug = self:RegisterCTRaidTrackerLoot( player, link, lootTypeID, lootGP );
        self:Debug('after cttracker register' .. (debug or 'nil'))
        
        self:Print( format(lootType.TEXT, player or 'nil', link or 'nil', lootGP or '', debug or '') );
	else
		self:Print( format("CDRCV(%s): %s", tostring(command), tostring(message) ) )
	end
end

function LootMaster:RegisterCTRaidTrackerLoot( player, link, lootTypeID, lootGP )
    
    local _,_,itemID = strfind(link, 'Hitem:(%d+)');
    if not itemID then return ' (Invalid link)' end;
    
    if not CT_RaidTracker_RaidLog then return '' end
    if not CT_RaidTracker_GetCurrentRaid
            or not CT_RaidTracker_RaidLog[CT_RaidTracker_GetCurrentRaid]
            or not CT_RaidTracker_RaidLog[CT_RaidTracker_GetCurrentRaid]["Loot"] then
        return ' (Unable to register in CT_RaidTracker; no raid started)'
    end
    
    for index, data in ipairs( CT_RaidTracker_RaidLog[CT_RaidTracker_GetCurrentRaid]["Loot"] ) do
        if data.player == player and strsplit(':', data.item.id) == itemID then
            
            if not lootGP or tonumber(lootGP)<=0 then lootGP=0 end;
            
            if lootTypeID == LootMaster.LOOTTYPE.BANK then
                CT_RaidTracker_RaidLog[CT_RaidTracker_GetCurrentRaid]["Loot"][index]['player'] = 'bank';
                lootGP = nil;
            elseif lootTypeID == LootMaster.LOOTTYPE.DISENCHANT then
                CT_RaidTracker_RaidLog[CT_RaidTracker_GetCurrentRaid]["Loot"][index]['player'] = 'disenchanted';
                lootGP = nil;
            end;            
            
            CT_RaidTracker_RaidLog[CT_RaidTracker_GetCurrentRaid]["Loot"][index]['costs'] = lootGP;
            return ' (Loot registered in CT_RaidTracker)'
        end
    end
    
    return ' (Loot not registered in CT_RaidTracker; please set it manually)'
end

function LootMaster:HasLoot( link )
    for i, data in ipairs(self.lootList) do repeat
        if not data then break end;
        if data.link == link then
            return i;
        end
    until true end
    return nil;
end

function LootMaster:GetLoot( link )
    local index = self:HasLoot(link);
    if not index then return nil end;
    
    return self.lootList[index]
end

function LootMaster:RemoveLoot( link )
    local index = self:HasLoot(link);
    if not index then return false end;
    
    tremove(self.lootList, index);
    
    return true;
end

function LootMaster:SendItemWanted( lootmaster, itemLink, response )    
    self:SendCommand("WANT", format("%s^%s^%s", itemLink or '', response or 0, self:GetEnchantingSkill() or 0), lootmaster)    
end

function LootMaster:GetEnchantingSkill()
    local numSkills = GetNumSkillLines();
    for i=1, numSkills do
        local skillName, _, _, skillRank = GetSkillLineInfo(i);
        if skillName == 'Enchanting' then return skillRank; end;
    end
    return 0;
end

function GetGearByINVTYPE( INVTYPE )
	if not INVTYPE_Slots[INVTYPE] then return '' end;
	local ret = {}
	local slot = INVTYPE_Slots[INVTYPE];
   
	local item = GetInventoryItemLink("player",GetInventorySlotInfo(slot[1] or slot))
	if not item and slot['or'] then
		item = GetInventoryItemLink("player",GetInventorySlotInfo(slot['or']))      
	end;
	if item then tinsert(ret, item) end;
	if slot[2] then
		item = GetInventoryItemLink("player",GetInventorySlotInfo(slot[2]))
		if item then tinsert(ret, item) end;
	end
    for i, item in ipairs(ret) do
        local _, _, _, _, _, _, _, _, _, itemTexture = GetItemInfo(item)
        local gpvalue, gpvalue2, ilevel = GetGPValue( item );
        ret[i] = format('%s^%s^%s^%s^%s', item, gpvalue or -1, ilevel or -1, gpvalue2 or -1, itemTexture)    
    end
	return strjoin('$', unpack(ret));   
end
--[[
	Kind of singleton, just create the frame once.
]]
function LootMaster:GetLootFrame()
	if self.lootframe then return self.lootframe end

	return 'test'
end


